<?php
/**
 * PointcutComponent.class
 * 
 * @package aowp.aspect.pointcut
 */
abstract class AOWP_PointcutComponent extends AOWP_Pointcut {
	private $_orPointcutArray;
	private $_andPointcutArray;
	
	abstract public function specificMatch(AOWP_JoinPoint $joinPoint);
	abstract public function runtimeSpecificMatch(AOWP_Context $context);
	
	/**
	 * Constructor.
	 */
	public function __construct() 
	{
		$this->_orPointcutArray  = array();
		$this->_andPointcutArray = array();
	}
	
	public function addAnd(AOWP_PointcutComponent $pointcut) 
	{
		$this->_andPointcutArray[] = $pointcut;
	}
	
	public function addOr(AOWP_PointcutComponent $pointcut) 
	{
		$this->_orPointcutArray[] = $pointcut;
	}
	
	public function Not() 
	{
		if ($this->isNotPointcut()) {
			$this->unsetNotFlag();
		} else {
			$this->setNotFlag();
		}
		
		if ($this->_isAndPointcutSet()) {
			foreach ($this->_addPointcutArray as $pointcut) {
				if ($pointcut->isNotPointcut()) {
					$pointcut->unsetNotFlag();
				} else {
					$pointcut->setNotFlag();
				}
			}
		}
		
		if ($this->_isOrPointcutSet()) {
			foreach ($this->_orPointcutArray as $pointcut) {
				if ($pointcut->isNotPointcut()) {
					$pointcut->unsetNotFlag();
				} else {
					$pointcut->setNotFlag();
				}
			}
		 }
	}
	
	public function isSpecificMatch($joinPoint)
	{
		$res = true;
		if (!$this->isNotPointcut() && !$this->specificMatch($joinPoint)) {
			$res = false;
		}
		if ($this->isNotPointcut() && $this->specificMatch($joinPoint)) {
			$res = false;
		} 
		return $res;
	}
	
	public function match(AOWP_JoinPoint $joinPoint) 
	{
		if (!$this->_isAndPointcutSet() && !$this->_isOrPointcutSet()) {
			if (!$this->isNotPointcut() && !$this->specificMatch($joinPoint)) {
				return false;
			}
			if ($this->isNotPointcut() && $this->specificMatch($joinPoint)) {
				return false;
			}
			return true;
		}
		
		$res = $this->isSpecificMatch($joinPoint);
		if ($this->_isOrPointcutSet()) {
			foreach ($this->_orPointcutArray as $pointcut) {
				$res = $res || ($pointcut->match($joinPoint));
			}
		}
		if ($this->_isAndPointcutSet()) {
			foreach ($this->_andPointcutArray as $pointcut) {
				$res = $res && ($pointcut->match($joinPoint));
			}
		}
		
		return $res;
	}
	
	public function isRuntimeSpecificMatch($context)
	{
		$res = true;
		if (!$this->isNotPointcut() && !$this->runtimeSpecificMatch($context)) {
			$res = false;
		}
		if ($this->isNotPointcut() && $this->runtimeSpecificMatch($context)) {
			$res = false;
		} 
		return $res;
	}
	
	public function runtimeMatch(AOWP_Context $context)
	{
		if (!$this->_isAndPointcutSet() && !$this->_isOrPointcutSet()) {
			if (!$this->isNotPointcut() && !$this->runtimeSpecificMatch($context)) {
				return false;
			}
			if ($this->isNotPointcut() && $this->runtimeSpecificMatch($context)) {
				return false;
			}
			return true;
		}
		
		$res = $this->isRuntimeSpecificMatch($context);
		if ($this->_isOrPointcutSet()) {
			foreach ($this->_orPointcutArray as $pointcut) {
				$res = $res || ($pointcut->runtimeMatch($context));
			}
		}
		if ($this->_isAndPointcutSet()) {
			foreach ($this->_andPointcutArray as $pointcut) {
				$res = $res && ($pointcut->runtimeMatch($context));
			}
		}
		
		return $res;
	}
	
	private function _isAndPointcutSet() 
	{
		return count($this->_andPointcutArray) > 0;
	}
	
	private function _isOrPointcutSet() 
	{
		return count($this->_orPointcutArray) > 0;
	}
}
?>
