<?php
/**
 * 
 * @package aowp.aspect.pointcut.pagehistory
 */

/**
 * ページ遷移履歴を取り扱うポイントカット記述子。
 * 
 * @uses AOWP_RequestObserveAspect
 * @package aowp.aspect.pointcut.pagehistory
 */
class AOWP_PageHisoryPointcut extends AOWP_PointcutComponent implements AOWP_RichPointcut {
	
	private $_uniqueName;
	private $_observeRequestArray;
	private $_historyJustBeforeFlag;
	
	/**
	 * コンストラクタ
	 *
	 * @param string $uniqueName 履歴保存用のアスペクトの名前の定義に利用する。他のPageHistoryPointcutと重複しない名前をつける必要が有る。 
	 * @param array $observeRequestArray 監視に利用するページ遷移のパターンを表すRequestPointcutの配列。
	 * @param boolean $historyJustBeforeFlag 指定したページ遷移の履歴が直前の履歴である必要がある場合trueにする。
	 */
	public function __construct($uniqueName, $observeRequestArray = array(), $historyJustBeforeFlag = false) {
		$this->_uniqueName = $uniqueName;
		$this->_observeRequestArray = $observeRequestArray;
		$this->_historyJustBeforeFlag = $historyJustBeforeFlag;
		$this->_generateObserveAspectDefinition();
		
		array(new AOWP_RequestPointcut('index.php'), new AOWP_RequestPointcut('.*'), new AOWP_RequestPointcut('regist.php'));
	}
	private function _generateObserveAspectDefinition() {
		AOWP_AspectManager::saveWeaverGenerateAspect(
			$this->getObserveAspectName(),
			AOWP_RequestObserveAspect::getRequestObserveAspectSource($this->getObserveAspectName(), $this->_observeRequestArray, $this->_historyJustBeforeFlag));
	}
	
	public function addRequest(AOWP_RequestPointcut $requestPointcut, $contextName = null) {
		$this->_observeRequestArray[] = $requestPointcut;
		$this->_generateObserveAspectDefinition();
	}
	public function setHistoryJustBeforeFlag($historyJustBeforeFlag) {
		$this->_historyJustBeforeFlag = $historyJustBeforeFlag;
	}
	
	public function getObserveAspectName() {
		return $this->_uniqueName . '_RequestObserveAspect';
	}
	
	/**
	 * マッチしたページ遷移履歴の各コンテキスト情報を取り出す。
	 *
	 * @param number $historyIndex 例えば、A-->B-->Cのページ遷移パターンのとき、1でBのページ遷移の際のコンテキスト情報が取得できる。
	 * @return AOWP_Context
	 */
	public static function getContext($contextName) {
		$observeAspect = AOWP_AspectInstanceManager::getInstance($this->getObserveAspectName());
		return $observeAspect->getHistoryContext($contextName);
	}
	
	public function getHistoryContextSource($historyIndex) {
		return '';
	}
	
	public function getAdditonalAspects() {
		return array($this->getObserveAspectName());
	}
	
	public function specificMatch(AOWP_JoinPoint $joinPoint) {
		return true;
	}
	public function runtimeSpecificMatch(AOWP_Context $context) {
		$observeAspect = AOWP_AspectInstanceManager::getInstance($this->getObserveAspectName());
		return $observeAspect->isHistoryMatch();
	}
		
}
?>