<?php
class AOWP_ASTManager {
	const AST_NAME_SEPARTOR = '__';
	
	private $_RemainingASTTasks = array();
	
	const AST_CACHE_DIR = 'ASTCache';
	const SERIARIZE_FILENAME = 'AOWP_ASTManager';
	
	public function __construct() {
		AOWP_Logger::setStartTime();
		$this->_RemainingASTTasks = AOWP_PHPFileManager::getAllProjectPHPFile();
		AOWP_Logger::logging('[Instantiation of ASTManager] Load ' . count($this->_RemainingASTTasks) . ' file paths are loaded.');
	}
	
	public function getRemainingASTTaskNum() {
		return count($this->_RemainingASTTasks);
	}
	
	public static function cacheDirPath() {
		return AOWP_Configuration::WEAVING_DATA_FOLDER . DIRECTORY_SEPARATOR . AOWP_ASTManager::AST_CACHE_DIR;
	}
	private static function weavingCacheDirPath() {
		return AOWP_Configuration::WEAVING_DATA_FOLDER . DIRECTORY_SEPARATOR . AOWP_ASTManager::AST_CACHE_DIR . DIRECTORY_SEPARATOR . 'weave';
	}
	private static function getASTCacheName($filePathOrASTFileName) {
		$projectPathPos = strpos($filePathOrASTFileName, AOWP_Configuration::PROJECT_HOME_PATH);
		if ($projectPathPos !== false) {
			$relativePath = substr($filePathOrASTFileName, $projectPathPos + strlen(AOWP_Configuration::PROJECT_HOME_PATH));
			return str_replace(DIRECTORY_SEPARATOR, AOWP_ASTManager::AST_NAME_SEPARTOR, $relativePath) . '.ast';
		}
		else {
			return $filePathOrASTFileName;
		}
	}
	private static function getSourceRelativePath($astFileName) {
		$sourcePath = substr($astFileName, 0, strlen($astFileName) - strlen('.ast'));
		return str_replace(AOWP_ASTManager::AST_NAME_SEPARTOR, DIRECTORY_SEPARATOR, $sourcePath);
	}
	private static function getSourcePath($astFileName) {
		return AOWP_Configuration::PROJECT_HOME_PATH . AOWP_ASTManager::getSourceRelativePath($astFileName);
	}
	private static function getWovenASTPath($filePathOrASTFileName) {
		return AOWP_ASTManager::weavingCacheDirPath() . DIRECTORY_SEPARATOR . AOWP_ASTManager::getASTCacheName($filePathOrASTFileName);
	}
	private static function getASTPath($filePathOrASTFileName) {
		return AOWP_ASTManager::cacheDirPath() . DIRECTORY_SEPARATOR . AOWP_ASTManager::getASTCacheName($filePathOrASTFileName);
	}
	
	public static function saveCache($fileName, $ast, $cacheDir) {
		if (!file_exists($cacheDir)) {
			AOWP_PHPFileManager::mkdir($cacheDir);
		}
		$savedFileName = preg_match('/.ast$/', $fileName) ? $fileName : AOWP_ASTManager::getASTCacheName($fileName); 
		$astCacheContent = serialize($ast);
		file_put_contents($cacheDir . DIRECTORY_SEPARATOR . $savedFileName, $astCacheContent);
	}
	public static function saveWeaveCache($fileName, $ast) {
		AOWP_ASTManager::saveCache($fileName, $ast, AOWP_ASTManager::weavingCacheDirPath());
	}
	
	
	public static function getAllCachedASTFileName() {
		$astPathArray = array();
		$astDirectory = dir(AOWP_ASTManager::cacheDirPath());
		while (($file = $astDirectory->read()) !== false) {
			if (!is_dir($file) && preg_match('/.ast$/', $file)) {
				$astPathArray[] = $file;
			}
		}
		return $astPathArray;
	}
	
	public static function parsePHPFile($fileName) {
		$cacheDir = AOWP_ASTManager::cacheDirPath();
		if (!file_exists($cacheDir)) {
			AOWP_PHPFileManager::mkdir($cacheDir);
		}	
		$cacheFilePath = AOWP_ASTManager::getASTPath($fileName);
		$ast = null;
		AOWP_Logger::setStartTime();
		if (file_exists($cacheFilePath)) {
			$cacheFileContent = file_get_contents($cacheFilePath);
			$ast = unserialize($cacheFileContent);
			AOWP_Logger::logging('[Use AST cache] ' . $fileName);
		}
		else {
			$ast = PHP_Parser::staticParseFile($fileName);
			AOWP_ASTManager::saveCache($fileName, $ast, $cacheDir);
			AOWP_Logger::logging('[Parse PHP file] ' . $fileName);
		}
		return $ast;
	}
	
	public function createAllASTCache() {
		if (file_exists(AOWP_ASTManager::cacheDirPath())) {
			AOWP_PHPFileManager::removeFolder(AOWP_ASTManager::cacheDirPath());
			AOWP_Logger::logging('[Creat AST cache] Reflesh cache directory.');
		}
		while(!$this->isEndCreateCache()) {
			$this->createOneASTCache();
		}
	}
	public function createOneASTCache(){
		$filepath = array_shift($this->_RemainingASTTasks);
		AOWP_ASTManager::parsePHPFile($filepath);
	}
	
	public function isEndCreateCache(){
		return $this->_RemainingASTTasks === array();
	}
	
	
	public function cleanASTCache(){
		$cacheDir = AOWP_Configuration::WEAVING_DATA_DIR_PATH . DIRECTORY_SEPARATOR . AOWP_ASTManager::AST_CACHE_DIR;
		if(file_exists($cacheDir)){
			$directory = dir($cacheDir);
			while (($fileName = $directory->read()) !== false) {
				if(is_file($fileName) && preg_match('/\.ast$/', $fileName)){
					unlink($fileName);
				}
			}
		}
	}
	
	public function getSerializeFilePath(){
		return AOWP_Configuration::WEAVING_DATA_FOLDER . DIRECTORY_SEPARATOR . AOWP_ASTManager::SERIARIZE_FILENAME;
	}
	
	public function serialize(){
		$serializeFilePath = $this->getSerializeFilePath();
		$seriarizeContent = serialize($this);
		file_put_contents($serializeFilePath, $seriarizeContent);
	}
	
	public static function generateWovenSource() {
		AOWP_PHPFileManager::backupOriginalPHPFile();
		if (!file_exists(AOWP_Configuration::PROJECT_HOME_PATH)) {
			AOWP_PHPFileManager::mkdir(AOWP_Configuration::PROJECT_HOME_PATH);
		}
		$allASTFileNameArray = AOWP_ASTManager::getAllCachedASTFileName();
		foreach ($allASTFileNameArray as $astFileName) {
			AOWP_Logger::setStartTime();
			$ast = unserialize(file_get_contents(AOWP_ASTManager::getWovenASTPath($astFileName)));
			$sourcePath = AOWP_ASTManager::getSourcePath($astFileName);
			if (!file_exists(dirname($sourcePath))) {
				AOWP_PHPFileManager::mkdir(dirname($sourcePath));
			}
			if (file_put_contents($sourcePath, AOWP_TemplateEngine::scriptToSource($ast->root))) {
				AOWP_Logger::logging('[Woven AST to woven source] Save source ' . $sourcePath);
			}
			else {
				AOWP_Logger::logging('[Woven AST to woven source] Fail in conversion to source ' . $sourcePath);
				exit();
			}
		}
	}
}

?>